@@ -7,6 +7,9 @@ import sys
 from disorder.graphical.model import Model
 from disorder.graphical.view import View
 from disorder.graphical.presenter import Presenter
+from disorder.graphical.utilities import report_exception
+
+from disorder.diffuse.scattering import parallelism
 
 class Window:
 
@@ -17,10 +20,12 @@ class Window:
   def show(self):
       self.view.show()
      
-def run():
+def run(): 
     app = QtWidgets.QApplication(sys.argv)
+    parallelism()
     view = View()
     adder = Presenter(Model(), view)
     window = Window(view, adder)
     window.show()    
+    sys.excepthook = report_exception
     sys.exit(app.exec_())
\ No newline at end of file
